//! This file is part of Flight.
//! 
//! Flight is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//! 
//! Flight is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//! You should have received a copy of the GNU General Public License along with Flight. If not, see <https://www.gnu.org/licenses/>.

// FFI for GLES 3.

#![allow(unused)]
#![allow(non_snake_case)]

use std::os::raw::{c_float, c_char, c_uchar, c_uint, c_int, c_void};
use super::ffi_sdl::{SDL_GLContext, SDL_GL_GetProcAddress};

pub type GLvoid = c_void;
pub type GLboolean = c_uchar;
pub type GLbyte = c_char;
pub type GLubyte = c_uchar;
pub type GLchar = c_char;
pub type GLuchar = c_uchar;
pub type GLbitfield = c_uint;
pub type GLenum = c_uint;
pub type GLint = c_int;
pub type GLuint = c_uint;
pub type GLsizei = c_int;
pub type GLintptr = isize;
pub type GLsizeiptr = isize;
pub type GLfloat = c_float;

pub const GL_DEPTH_BUFFER_BIT: GLbitfield = 0x00000100;
pub const GL_STENCIL_BUFFER_BIT: GLbitfield = 0x00000400;
pub const GL_COLOR_BUFFER_BIT: GLbitfield = 0x00004000;

pub const GL_ARRAY_BUFFER: GLenum = 0x8892;
pub const GL_ELEMENT_ARRAY_BUFFER: GLenum = 0x8893;
pub const GL_ARRAY_BUFFER_BINDING: GLenum = 0x8894;
pub const GL_ELEMENT_ARRAY_BUFFER_BINDING: GLenum = 0x8895;

pub const GL_STREAM_DRAW: GLenum = 0x88E0;
pub const GL_STATIC_DRAW: GLenum = 0x88E4;
pub const GL_DYNAMIC_DRAW: GLenum = 0x88E8;

pub const GL_FRAGMENT_SHADER: GLenum = 0x8B30;
pub const GL_VERTEX_SHADER: GLenum = 0x8B31;

pub const GL_NO_ERROR: GLenum = 0;
pub const GL_INVALID_ENUM: GLenum = 0x0500;
pub const GL_INVALID_VALUE: GLenum = 0x0501;
pub const GL_INVALID_OPERATION: GLenum = 0x0502;
pub const GL_INVALID_FRAMEBUFFER_OPERATION: GLenum = 0x0506;
pub const GL_OUT_OF_MEMORY: GLenum = 0x0505;

pub const GL_FALSE: GLboolean = 0;
pub const GL_TRUE: GLboolean = 1;

pub const GL_BYTE: GLenum = 0x1400;
pub const GL_UNSIGNED_BYTE: GLenum = 0x1401;
pub const GL_SHORT: GLenum = 0x1402;
pub const GL_UNSIGNED_SHORT: GLenum = 0x1403;
pub const GL_INT: GLenum = 0x1404;
pub const GL_UNSIGNED_INT: GLenum = 0x1405;
pub const GL_FLOAT: GLenum = 0x1406;
pub const GL_FIXED: GLenum = 0x140C;

pub const GL_POINTS: GLenum = 0x0000;
pub const GL_LINES: GLenum = 0x0001;
pub const GL_LINE_LOOP: GLenum = 0x0002;
pub const GL_LINE_STRIP: GLenum = 0x0003;
pub const GL_TRIANGLES: GLenum = 0x0004;
pub const GL_TRIANGLE_STRIP: GLenum = 0x0005;
pub const GL_TRIANGLE_FAN: GLenum = 0x0006;

pub const GL_PRIMITIVE_RESTART_FIXED_INDEX: GLenum = 0x8D69;

pub const GL_COMPILE_STATUS: GLenum = 0x8B81;
pub const GL_LINK_STATUS: GLenum = 0x8B82;
pub const GL_INFO_LOG_LENGTH: GLenum = 0x8B84;
pub const GL_CURRENT_PROGRAM: GLenum = 0x8B8D;

pub const GL_RED: GLenum = 0x1903;
pub const GL_GREEN: GLenum = 0x1904;
pub const GL_BLUE: GLenum = 0x1905;
pub const GL_ALPHA: GLenum = 0x1906;
pub const GL_RGB: GLenum = 0x1907;
pub const GL_RGBA: GLenum = 0x1908;

pub const GL_ALPHA8: GLint = 0x803C;
pub const GL_RGB8: GLint = 0x8051;
pub const GL_RGBA8: GLint = 0x8058;

pub const GL_TEXTURE_1D: GLenum = 0x0DE0;
pub const GL_TEXTURE_2D: GLenum = 0x0DE1;
pub const GL_TEXTURE_WRAP_S: GLenum = 0x2802;
pub const GL_TEXTURE_WRAP_T: GLenum = 0x2803;
pub const GL_TEXTURE_MAG_FILTER: GLenum = 0x2800;
pub const GL_TEXTURE_MIN_FILTER: GLenum = 0x2801;
pub const GL_NEAREST_MIPMAP_NEAREST: GLenum = 0x2700;
pub const GL_NEAREST_MIPMAP_LINEAR: GLenum = 0x2702;
pub const GL_LINEAR_MIPMAP_NEAREST: GLenum = 0x2701;
pub const GL_LINEAR_MIPMAP_LINEAR: GLenum = 0x2703;
pub const GL_NEAREST: GLint = 0x2600;
pub const GL_LINEAR: GLint = 0x2601;
pub const GL_REPEAT: GLint = 0x2901;
pub const GL_CLAMP_TO_EDGE: GLint = 0x812F;
pub const GL_MIRRORED_REPEAT: GLint = 0x8370;

pub const GL_TEXTURE0: GLenum = 0x84C0;

pub const GL_BLEND: GLenum = 0x0BE2;
pub const GL_BLEND_SRC: GLenum = 0x0BE1;
pub const GL_BLEND_DST: GLenum = 0x0BE0;
pub const GL_ZERO: GLenum = 0;
pub const GL_ONE: GLenum = 1;
pub const GL_SRC_COLOR: GLenum = 0x0300;
pub const GL_ONE_MINUS_SRC_COLOR: GLenum = 0x0301;
pub const GL_SRC_ALPHA: GLenum = 0x0302;
pub const GL_ONE_MINUS_SRC_ALPHA: GLenum = 0x0303;
pub const GL_DST_ALPHA: GLenum = 0x0304;
pub const GL_ONE_MINUS_DST_ALPHA: GLenum = 0x0305;
pub const GL_DST_COLOR: GLenum = 0x0306;
pub const GL_ONE_MINUS_DST_COLOR: GLenum = 0x0307;
pub const GL_SRC_ALPHA_SATURATE: GLenum = 0x0308;

pub const GL_FRONT: GLenum = 0x0404;
pub const GL_BACK: GLenum = 0x0405;
pub const GL_FRONT_AND_BACK: GLenum = 0x0408;

pub const GL_CULL_FACE_MODE: GLenum = 0x0B45;

pub const GL_DEPTH_TEST: GLenum = 0x0B71;
pub const GL_STENCIL_TEST: GLenum = 0x0B90;

pub const GL_DEPTH_FUNC: GLenum = 0x0B74;

pub const GL_STENCIL_BITS: GLenum = 0x0D57;

pub const GL_NEVER: GLenum = 0x0200;
pub const GL_LESS: GLenum = 0x0201;
pub const GL_EQUAL: GLenum = 0x0202;
pub const GL_LEQUAL: GLenum = 0x0203;
pub const GL_GREATER: GLenum = 0x0204;
pub const GL_NOTEQUAL: GLenum = 0x0205;
pub const GL_GEQUAL: GLenum = 0x0206;
pub const GL_ALWAYS: GLenum = 0x0207;

pub const GL_KEEP: GLenum = 0x1E00;
pub const GL_REPLACE: GLenum = 0x1E01;
pub const GL_INCR: GLenum = 0x1E02;
pub const GL_DECR: GLenum = 0x1E03;
pub const GL_INVERT: GLenum = 0x150A;
pub const GL_INCR_WRAP: GLenum = 0x8507;
pub const GL_DECR_WRAP: GLenum = 0x8508;

pub const GL_CULL_FACE: GLenum = 0x0B44;

/// Macro to generate a `struct` with a field for every OpenGL function we want to load.
/// Also generate a method to automatically dynamically load every function using SDL.
macro_rules! gl_struct {
	($($fn:ident : fn($($arg:ty),*) -> $return:ty),+ $(,)*) => {
		pub struct GL {
			$(pub $fn: unsafe extern "system" fn($($arg),*) -> $return,)+
		}
		impl GL {
			pub unsafe fn load(_context: *mut SDL_GLContext) -> Result<Self, String> {
				Ok(Self {
					$($fn: {
						let name = concat!("gl", stringify!($fn), "\0").as_ptr() as *const c_char;
						let raw = SDL_GL_GetProcAddress(name);
						if raw.is_null() { return Err(format!("failed to load OpenGL function {}", stringify!($fn))); }
						assert!(!raw.is_null());
						std::mem::transmute::<*mut c_void, extern "system" fn($($arg),*) -> $return>(raw)
					},)*
				})
			}
			// Check for OpenGL errors.
			pub fn check(&self) {
				if cfg!(debug_assertions) {
					match unsafe{(self.GetError)()} {
						GL_NO_ERROR => (),
						GL_INVALID_ENUM => panic!("GLES: invalid enum"),
						GL_INVALID_VALUE => panic!("GLES: invalid value"),
						GL_INVALID_OPERATION => panic!("GLES: invalid operation"),
						GL_INVALID_FRAMEBUFFER_OPERATION => panic!("GLES: invalid framebuffer operation"),
						GL_OUT_OF_MEMORY => panic!("GLES: out of memory"),
						_ => panic!("GLES: unknown error"),
					}
				}
			}
		}
	}
}

gl_struct!(
	ActiveTexture: fn(GLenum) -> (),
	AttachShader: fn(GLuint, GLuint) -> (),
	BindAttribLocation: fn(GLuint, GLuint, *const GLchar) -> (),
	BindBuffer: fn(GLenum, GLuint) -> (),
	BindTexture: fn(GLenum, GLuint) -> (),
	BlendFunc: fn(GLenum, GLenum) -> (),
	BufferData: fn(GLenum, GLsizeiptr, *const GLvoid, GLenum) -> (),
	Clear: fn(GLbitfield) -> (),
	ClearColor: fn(GLfloat, GLfloat, GLfloat, GLfloat) -> (),
	ClearStencil: fn(GLint) -> (),
	ColorMask: fn(GLboolean, GLboolean, GLboolean, GLboolean) -> (),
	CompileShader: fn(GLuint) -> (),
	CreateProgram: fn() -> GLuint,
	CreateShader: fn(GLenum) -> GLuint,
	CullFace: fn(GLenum) -> (),
	DeleteBuffers: fn(GLsizei, *const GLuint) -> (),
	DeleteProgram: fn(GLuint) -> (),
	DeleteShader: fn(GLuint) -> (),
	DeleteTextures: fn(GLsizei, *const GLuint) -> (),
	DepthFunc: fn(GLenum) -> (),
	DepthMask: fn(GLboolean) -> (),
	Disable: fn(GLenum) -> (),
	DrawElements: fn(GLenum, GLsizei, GLenum, *const GLvoid) -> (),
	Enable: fn(GLenum) -> (),
	EnableVertexAttribArray: fn(GLuint) -> (),
	GenBuffers: fn(GLsizei, *mut GLuint) -> (),
	GenTextures: fn(GLsizei, *mut GLuint) -> (),
	GenerateMipmap: fn(GLenum) -> (),
	GetAttribLocation: fn(GLuint, *const GLchar) -> GLint,
	GetBooleanv: fn(GLenum, *mut GLboolean) -> (),
	GetError: fn() -> GLenum,
	GetIntegerv: fn(GLenum, *mut GLint) -> (),
	GetProgramiv: fn(GLuint, GLenum, *mut GLint) -> (),
	GetProgramInfoLog: fn(GLuint, GLsizei, *mut GLsizei, *mut GLchar) -> (),
	GetShaderiv: fn(GLuint, GLenum, *mut GLint) -> (),
	GetShaderInfoLog: fn(GLuint, GLsizei, *mut GLsizei, *mut GLchar) -> (),
	GetUniformLocation: fn(GLuint, *const GLchar) -> GLint,
	LinkProgram: fn(GLuint) -> (),
	ShaderSource: fn(GLuint, GLsizei, *const *const GLchar, *const GLint) -> (),
	StencilFunc: fn(GLenum, GLint, GLuint) -> (),
	StencilFuncSeparate: fn(GLenum, GLenum, GLint, GLuint) -> (),
	StencilOp: fn(GLenum, GLenum, GLenum) -> (),
	StencilOpSeparate: fn(GLenum, GLenum, GLenum, GLenum) -> (),
	TexImage2D: fn(GLenum, GLint, GLint, GLsizei, GLsizei, GLint, GLenum, GLenum, *const GLvoid) -> (),
	TexParameteri: fn(GLenum, GLenum, GLint) -> (),
	Uniform1f: fn(GLint, GLfloat) -> (),
	Uniform2f: fn(GLint, GLfloat, GLfloat) -> (),
	Uniform3f: fn(GLint, GLfloat, GLfloat, GLfloat) -> (),
	Uniform4f: fn(GLint, GLfloat, GLfloat, GLfloat, GLfloat) -> (),
	Uniform1i: fn(GLint, GLint) -> (),
	UniformMatrix3fv: fn(GLint, GLsizei, GLboolean, *const GLfloat) -> (),
	UseProgram: fn(GLuint) -> (),
	ValidateProgram: fn(GLuint) -> (),
	Viewport: fn(GLint, GLint, GLsizei, GLsizei) -> (),
	VertexAttribPointer: fn(GLuint, GLint, GLenum, GLboolean, GLsizei, *const GLvoid) -> (),
);
