//! This file is part of Flight.
//! 
//! Flight is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//! 
//! Flight is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//! You should have received a copy of the GNU General Public License along with Flight. If not, see <https://www.gnu.org/licenses/>.

//! Basic FFI for SDL2.

#![allow(unused)]
#![allow(non_snake_case)]
#![allow(non_camel_case_types)]

use std::os::raw::{c_char, c_double, c_float, c_int, c_void};
use super::ffi_keycodes::*;

pub const SDL_INIT_TIMER: u32 = 0x00000001;
pub const SDL_INIT_AUDIO: u32 = 0x00000010;
pub const SDL_INIT_VIDEO: u32 = 0x00000020;
pub const SDL_INIT_JOYSTICK: u32 = 0x00000200;
pub const SDL_INIT_GAMECONTROLLER: u32 = 0x00002000;
pub const SDL_INIT_EVENTS: u32 = 0x00004000;

pub const SDL_WINDOW_FULLSCREEN: u32 = 0x00000001;
pub const SDL_WINDOW_OPENGL: u32 = 0x00000002;
pub const SDL_WINDOW_RESIZABLE: u32 = 0x00000020;
pub const SDL_WINDOW_MAXIMIZED: u32 = 0x00000080;
pub const SDL_WINDOW_FULLSCREEN_DESKTOP: u32 = SDL_WINDOW_FULLSCREEN | 0x00001000;
pub const SDL_WINDOW_FOREIGN: u32 = 0x00000800;
pub const SDL_WINDOW_ALLOW_HIGHDPI: u32 = 0x00002000;

pub const SDL_WINDOWPOS_UNDEFINED: c_int = 0x1FFF0000;

pub const SDL_QUIT: u32 = 0x100;
pub const SDL_APP_TERMINATING: u32 = 0x101;
pub const SDL_APP_LOWMEMORY: u32 = 0x102;
pub const SDL_APP_WILLENTERBACKGROUND: u32 = 0x103;
pub const SDL_APP_DIDENTERBACKGROUND: u32 = 0x104;
pub const SDL_APP_WILLENTERFOREGROUND: u32 = 0x105;
pub const SDL_APP_DIDENTERFOREGROUND: u32 = 0x106;
pub const SDL_DISPLAYEVENT: u32 = 0x150;
pub const SDL_WINDOWEVENT: u32 = 0x200;
pub const SDL_SYSWMEVENT: u32 = 0x201;
pub const SDL_KEYDOWN: u32 = 0x300;
pub const SDL_KEYUP: u32 = 0x301;
pub const SDL_TEXTEDITING: u32 = 0x302;
pub const SDL_TEXTINPUT: u32 = 0x303;
pub const SDL_KEYMAPCHANGED: u32 = 0x303;
pub const SDL_MOUSEMOTION: u32 = 0x400;
pub const SDL_MOUSEBUTTONDOWN: u32 = 0x401;
pub const SDL_MOUSEBUTTONUP: u32 = 0x402;
pub const SDL_MOUSEWHEEL: u32 = 0x403;
pub const SDL_JOYAXISMOTION: u32 = 0x600;
pub const SDL_JOYBALLMOTION: u32 = 0x601;
pub const SDL_JOYHATMOTION: u32 = 0x602;
pub const SDL_JOYBUTTONDOWN: u32 = 0x603;
pub const SDL_JOYBUTTONUP: u32 = 0x604;
pub const SDL_JOYDEVICEADDED: u32 = 0x605;
pub const SDL_JOYDEVICEREMOVED: u32 = 0x606;
pub const SDL_CONTROLLERAXISMOTION: u32 = 0x650;
pub const SDL_CONTROLLERBUTTONDOWN: u32 = 0x651;
pub const SDL_CONTROLLERBUTTONUP: u32 = 0x652;
pub const SDL_CONTROLLERDEVICEADDED: u32 = 0x653;
pub const SDL_CONTROLLERDEVICEREMOVED: u32 = 0x654;
pub const SDL_CONTROLLERDEVICEREMAPPED: u32 = 0x655;
pub const SDL_FINGERDOWN: u32 = 0x700;
pub const SDL_FINGERUP: u32 = 0x701;
pub const SDL_FINGERMOTION: u32 = 0x702;
pub const SDL_DOLLARGESTURE: u32 = 0x800;
pub const SDL_DOLLARRECORD: u32 = 0x801;
pub const SDL_MULTIGESTURE: u32 = 0x802;
pub const SDL_CLIPBOARDUPDATE: u32 = 0x900;
pub const SDL_DROPFILE: u32 = 0x1000;
pub const SDL_DROPTEXT: u32 = 0x1001;
pub const SDL_DROPBEGIN: u32 = 0x1002;
pub const SDL_DROPCOMPLETE: u32 = 0x1003;
pub const SDL_AUDIODEVICEADDED: u32 = 0x1100;
pub const SDL_AUDIODEVICEREMOVED: u32 = 0x1101;
pub const SDL_SENSORUPDATE: u32 = 0x1200;
pub const SDL_RENDER_TARGETS_RESET: u32 = 0x2000;
pub const SDL_RENDER_DEVICE_RESET: u32 = 0x2001;
pub const SDL_USEREVENT: u32 = 0x8000;

pub const SDL_BUTTON_LEFT: u8 = 1;
pub const SDL_BUTTON_MIDDLE: u8 = 2;
pub const SDL_BUTTON_RIGHT: u8 = 3;

pub const AUDIO_U8: u32 = 0x0008;
pub const AUDIO_S8: u32 = 0x8008;
pub const AUDIO_U16LSB: u32 = 0x0010;
pub const AUDIO_S16LSB: u32 = 0x8010;
pub const AUDIO_F32LSB: u32 = 0x8120;

pub const SDL_GL_CONTEXT_PROFILE_CORE: c_int = 0x0001;
pub const SDL_GL_CONTEXT_PROFILE_COMPATIBILITY: c_int = 0x0002;
pub const SDL_GL_CONTEXT_PROFILE_ES: c_int = 0x0004;

pub const SDL_GL_CONTEXT_DEBUG_FLAG: c_int = 0x0001;
pub const SDL_GL_CONTEXT_FORWARD_COMPATIBLE_FLAG: c_int = 0x0002;
pub const SDL_GL_CONTEXT_ROBUST_ACCESS_FLAG: c_int = 0x0004;
pub const SDL_GL_CONTEXT_RESET_ISOLATION_FLAG: c_int = 0x0008;

pub const SDL_PIXELFORMAT_RGBA8888: u32 = 0x16462004;
pub const SDL_PIXELFORMAT_RGBX8888: u32 = 0x16261804;
pub const SDL_PIXELFORMAT_RGB888: u32 = 0x16161804;

#[repr(C)] pub struct SDL_Window { _private: [u8; 0] }
#[repr(C)] pub struct SDL_GLContext { _private: [u8; 0] }
#[repr(C)] pub struct SDL_Palette { _private: [u8; 0] }
#[repr(C)] pub struct SDL_Joystick { _private: [u8; 0] }
#[repr(C)] pub struct SDL_GameController { _private: [u8; 0] }
#[repr(C)] pub struct SDL_RWops { _private: [u8; 0] }

#[repr(C)] #[derive(Copy, Clone)] pub struct SDL_PixelFormat {
	pub format: u32,
	pub palette: *mut SDL_Palette,
	pub BitsPerPixel: u8,
	pub BytesPerPixel: u8,
	pub padding: [u8; 2],
	pub Rmask: u32,
	pub Gmask: u32,
	pub Bmask: u32,
	pub Amask: u32,
	pub Rloss: u8,
	pub Gloss: u8,
	pub Bloss: u8,
	pub Aloss: u8,
	pub Rshift: u8,
	pub Gshift: u8,
	pub Bshift: u8,
	pub Ashift: u8,
	pub refcount: c_int,
	pub next: *mut Self,
}
#[repr(C)] #[derive(Copy, Clone)] pub struct SDL_Rect {
	pub x: c_int,
	pub y: c_int,
	pub w: c_int,
	pub h: c_int,
}
#[repr(C)] pub struct SDL_Surface {
	pub flags: u32,
	pub format: *mut SDL_PixelFormat,
	pub w: c_int,
	pub h: c_int,
	pub pitch: c_int,
	pub pixels: *mut c_void,
	pub userdata: *mut c_void,
	pub locked: c_int,
	pub lock_data: *mut c_void,
	pub clip_rect: SDL_Rect,
	pub map: *const c_void,
	pub refcount: c_int,
}
#[repr(C)] #[derive(Copy, Clone, Eq, PartialEq)] pub enum SDL_GameControllerAxis {
	SDL_CONTROLLER_AXIS_INVALID = -1,
	SDL_CONTROLLER_AXIS_LEFTX,
	SDL_CONTROLLER_AXIS_LEFTY,
	SDL_CONTROLLER_AXIS_RIGHTX,
	SDL_CONTROLLER_AXIS_RIGHTY,
	SDL_CONTROLLER_AXIS_TRIGGERLEFT,
	SDL_CONTROLLER_AXIS_TRIGGERRIGHT,
	SDL_CONTROLLER_AXIS_MAX
}
// Convenience aliases to make comparing with contents of events cleaner.
pub const SDL_CONTROLLER_AXIS_LEFTX: u8 = SDL_GameControllerAxis::SDL_CONTROLLER_AXIS_LEFTX as u8;
pub const SDL_CONTROLLER_AXIS_LEFTY: u8 = SDL_GameControllerAxis::SDL_CONTROLLER_AXIS_LEFTY as u8;
pub const SDL_CONTROLLER_AXIS_RIGHTX: u8 = SDL_GameControllerAxis::SDL_CONTROLLER_AXIS_RIGHTX as u8;
pub const SDL_CONTROLLER_AXIS_RIGHTY: u8 = SDL_GameControllerAxis::SDL_CONTROLLER_AXIS_RIGHTY as u8;
pub const SDL_CONTROLLER_AXIS_TRIGGERLEFT: u8 = SDL_GameControllerAxis::SDL_CONTROLLER_AXIS_TRIGGERLEFT as u8;
pub const SDL_CONTROLLER_AXIS_TRIGGERRIGHT: u8 = SDL_GameControllerAxis::SDL_CONTROLLER_AXIS_TRIGGERRIGHT as u8;
#[repr(C)] #[derive(Copy, Clone, Eq, PartialEq)] pub enum SDL_GameControllerButton {
	SDL_CONTROLLER_BUTTON_INVALID = -1,
	SDL_CONTROLLER_BUTTON_A,
	SDL_CONTROLLER_BUTTON_B,
	SDL_CONTROLLER_BUTTON_X,
	SDL_CONTROLLER_BUTTON_Y,
	SDL_CONTROLLER_BUTTON_BACK,
	SDL_CONTROLLER_BUTTON_GUIDE,
	SDL_CONTROLLER_BUTTON_START,
	SDL_CONTROLLER_BUTTON_LEFTSTICK,
	SDL_CONTROLLER_BUTTON_RIGHTSTICK,
	SDL_CONTROLLER_BUTTON_LEFTSHOULDER,
	SDL_CONTROLLER_BUTTON_RIGHTSHOULDER,
	SDL_CONTROLLER_BUTTON_DPAD_UP,
	SDL_CONTROLLER_BUTTON_DPAD_DOWN,
	SDL_CONTROLLER_BUTTON_DPAD_LEFT,
	SDL_CONTROLLER_BUTTON_DPAD_RIGHT,
	SDL_CONTROLLER_BUTTON_MAX
}
// Convenience aliases to make comparing with contents of events cleaner.
pub const SDL_CONTROLLER_BUTTON_A: u8 = SDL_GameControllerButton::SDL_CONTROLLER_BUTTON_A as u8;
pub const SDL_CONTROLLER_BUTTON_B: u8 = SDL_GameControllerButton::SDL_CONTROLLER_BUTTON_B as u8;
pub const SDL_CONTROLLER_BUTTON_X: u8 = SDL_GameControllerButton::SDL_CONTROLLER_BUTTON_X as u8;
pub const SDL_CONTROLLER_BUTTON_Y: u8 = SDL_GameControllerButton::SDL_CONTROLLER_BUTTON_Y as u8;
pub const SDL_CONTROLLER_BUTTON_BACK: u8 = SDL_GameControllerButton::SDL_CONTROLLER_BUTTON_BACK as u8;
pub const SDL_CONTROLLER_BUTTON_GUIDE: u8 = SDL_GameControllerButton::SDL_CONTROLLER_BUTTON_GUIDE as u8;
pub const SDL_CONTROLLER_BUTTON_START: u8 = SDL_GameControllerButton::SDL_CONTROLLER_BUTTON_START as u8;
pub const SDL_CONTROLLER_BUTTON_LEFTSTICK: u8 = SDL_GameControllerButton::SDL_CONTROLLER_BUTTON_LEFTSTICK as u8;
pub const SDL_CONTROLLER_BUTTON_RIGHTSTICK: u8 = SDL_GameControllerButton::SDL_CONTROLLER_BUTTON_RIGHTSTICK as u8;
pub const SDL_CONTROLLER_BUTTON_LEFTSHOULDER: u8 = SDL_GameControllerButton::SDL_CONTROLLER_BUTTON_LEFTSHOULDER as u8;
pub const SDL_CONTROLLER_BUTTON_RIGHTSHOULDER: u8 = SDL_GameControllerButton::SDL_CONTROLLER_BUTTON_RIGHTSHOULDER as u8;
pub const SDL_CONTROLLER_BUTTON_DPAD_UP: u8 = SDL_GameControllerButton::SDL_CONTROLLER_BUTTON_DPAD_UP as u8;
pub const SDL_CONTROLLER_BUTTON_DPAD_DOWN: u8 = SDL_GameControllerButton::SDL_CONTROLLER_BUTTON_DPAD_DOWN as u8;
pub const SDL_CONTROLLER_BUTTON_DPAD_LEFT: u8 = SDL_GameControllerButton::SDL_CONTROLLER_BUTTON_DPAD_LEFT as u8;
pub const SDL_CONTROLLER_BUTTON_DPAD_RIGHT: u8 = SDL_GameControllerButton::SDL_CONTROLLER_BUTTON_DPAD_RIGHT as u8;
#[repr(C)] #[derive(Copy, Clone)] pub struct SDL_AudioSpec {
	freq: c_int,
	format: u16,
	channels: u8,
	silence: u8,
	samples: u16,
	padding: u16,
	size: u32,
	callback: *const c_void,
	userdata: *mut c_void,
}
#[repr(C)] #[derive(Copy, Clone)] pub enum SDL_GLattr {
	SDL_GL_RED_SIZE,
	SDL_GL_GREEN_SIZE,
	SDL_GL_BLUE_SIZE,
	SDL_GL_ALPHA_SIZE,
	SDL_GL_BUFFER_SIZE,
	SDL_GL_DOUBLEBUFFER,
	SDL_GL_DEPTH_SIZE,
	SDL_GL_STENCIL_SIZE,
	SDL_GL_ACCUM_RED_SIZE,
	SDL_GL_ACCUM_GREEN_SIZE,
	SDL_GL_ACCUM_BLUE_SIZE,
	SDL_GL_ACCUM_ALPHA_SIZE,
	SDL_GL_STEREO,
	SDL_GL_MULTISAMPLEBUFFERS,
	SDL_GL_MULTISAMPLESAMPLES,
	SDL_GL_ACCELERATED_VISUAL,
	SDL_GL_RETAINED_BACKING,
	SDL_GL_CONTEXT_MAJOR_VERSION,
	SDL_GL_CONTEXT_MINOR_VERSION,
	SDL_GL_CONTEXT_EGL,
	SDL_GL_CONTEXT_FLAGS,
	SDL_GL_CONTEXT_PROFILE_MASK,
	SDL_GL_SHARE_WITH_CURRENT_CONTEXT,
	SDL_GL_FRAMEBUFFER_SRGB_CAPABLE,
	SDL_GL_CONTEXT_RELEASE_BEHAVIOR,
	SDL_GL_CONTEXT_RESET_NOTIFICATION,
	SDL_GL_CONTEXT_NO_ERROR
}
#[repr(u8)] #[derive(Copy, Clone, Eq, PartialEq)] pub enum WindowEvent {
	 None,
	 Shown,
	 Hidden,
	 Exposed,
	 Moved,
	 Resized,
	 SizeChanged,
	 Minimized,
	 Maximized,
	 Restored,
	 Enter,
	 Leave,
	 FocusGained,
	 FocusLost,
	 Close,
	 TakeFocus,
	 HitTest,
}
pub const KMOD_NONE: u16 = 0x0000;
pub const KMOD_LSHIFT: u16 = 0x0001;
pub const KMOD_RSHIFT: u16 = 0x0002;
pub const KMOD_LCTRL: u16 = 0x0040;
pub const KMOD_RCTRL: u16 = 0x0080;
pub const KMOD_LALT: u16 = 0x0100;
pub const KMOD_RALT: u16 = 0x0200;
pub const KMOD_LGUI: u16 = 0x0400;
pub const KMOD_RGUI: u16 = 0x0800;
pub const KMOD_NUM: u16 = 0x1000;
pub const KMOD_CAPS: u16 = 0x2000;
pub const KMOD_MODE: u16 = 0x4000;
pub const KMOD_RESERVED: u16 = 0x8000;
pub const KMOD_CTRL: u16 = (KMOD_LCTRL|KMOD_RCTRL);
pub const KMOD_SHIFT: u16 = (KMOD_LSHIFT|KMOD_RSHIFT);
pub const KMOD_ALT: u16 = (KMOD_LALT|KMOD_RALT);
pub const KMOD_GUI: u16 = (KMOD_LGUI|KMOD_RGUI);
#[repr(C)] #[derive(Copy, Clone)] pub struct SDL_Keysym {
	pub scancode: SDL_Scancode,
	pub sym: SDL_KeyCode,
	pub modifiers: u16,
	pub unused: u32,
}
impl SDL_Keysym {
	pub fn has_shift(&self) -> bool {
		self.modifiers & KMOD_SHIFT != 0
	}
	pub fn has_ctrl(&self) -> bool {
		self.modifiers & KMOD_CTRL != 0
	}
	pub fn has_alt(&self) -> bool {
		self.modifiers & KMOD_ALT != 0
	}
}
pub type SDL_AudioDeviceID = u32;
pub type SDL_JoystickID = i32;
#[repr(C)] #[derive(Copy, Clone)] pub struct SDL_DisplayEvent {
	tag: u32,
	timestamp: u32,
	display: u32,
	event: u8,
	padding1: u8,
	padding2: u8,
	padding3: u8,
	data1: i32,
}
#[repr(C)] #[derive(Copy, Clone)] pub struct SDL_WindowEvent {
	pub tag: u32,
	pub timestamp: u32,
	pub windowID: u32,
	pub event: WindowEvent,
	pub padding1: u8,
	pub padding2: u8,
	pub padding3: u8,
	pub data1: i32,
	pub data2: i32,
}
#[repr(C)] #[derive(Copy, Clone)] pub struct SDL_KeyboardEvent {
	pub tag: u32,
	pub timestamp: u32,
	pub windowID: u32,
	pub state: u8,
	pub repeat: u8,
	pub padding2: u8,
	pub padding3: u8,
	pub keysym: SDL_Keysym,
}
#[repr(C)] #[derive(Copy, Clone)] pub struct SDL_TextEditingEvent {
	pub tag: u32,
	pub timestamp: u32,
	pub windowID: u32,
	pub text: [u8; 32],
	pub start: i32,
	pub length: i32,
}
#[repr(C)] #[derive(Copy, Clone)] pub struct SDL_TextInputEvent {
	pub tag: u32,
	pub timestamp: u32,
	pub windowID: u32,
	pub text: [u8; 32],
}
#[repr(C)] #[derive(Copy, Clone)] pub struct SDL_MouseMotionEvent {
	pub tag: u32,
	pub timestamp: u32,
	pub windowID: u32,
	pub which: u32,
	pub state: u32,
	pub x: i32,
	pub y: i32,
	pub xrel: i32,
	pub yrel: i32,
}
#[repr(C)] #[derive(Copy, Clone)] pub struct SDL_MouseButtonEvent {
	pub tag: u32,
	pub timestamp: u32,
	pub windowID: u32,
	pub which: u32,
	pub button: u8,
	pub state: u8,
	pub clicks: u8,
	pub padding1: u8,
	pub x: i32,
	pub y: i32,
}
#[repr(C)] #[derive(Copy, Clone)] pub struct SDL_MouseWheelEvent {
	pub tag: u32,
	pub timestamp: u32,
	pub windowID: u32,
	pub which: u32,
	pub x: i32,
	pub y: i32,
	pub direction: u32,
}
#[repr(C)] #[derive(Copy, Clone)] pub struct SDL_ControllerAxisEvent {
	pub tag: u32,
	pub timestamp: u32,
	pub which: SDL_JoystickID,
	pub axis: u8,
	pub padding1: u8,
	pub padding2: u8,
	pub padding3: u8,
	pub value: i16,
	pub padding4: u16,
}
#[repr(C)] #[derive(Copy, Clone)] pub struct SDL_ControllerButtonEvent {
	pub tag: u32,
	pub timestamp: u32,
	pub which: SDL_JoystickID,
	pub button: u8,
	pub state: u8,
	pub padding1: u8,
	pub padding2: u8,
}
#[repr(C)] #[derive(Copy, Clone)] pub struct SDL_ControllerDeviceEvent {
	pub tag: u32,
	pub timestamp: u32,
	pub which: i32,
}
#[repr(C)] pub union SDL_Event {
	pub tag: u32,
	pub display: SDL_DisplayEvent,
	pub window: SDL_WindowEvent,
	pub key: SDL_KeyboardEvent,
	pub edit: SDL_TextEditingEvent,
	pub input: SDL_TextInputEvent,
	pub motion: SDL_MouseMotionEvent,
	pub button: SDL_MouseButtonEvent,
	pub wheel: SDL_MouseWheelEvent,
	pub caxis: SDL_ControllerAxisEvent,
	pub cbutton: SDL_ControllerButtonEvent,
	pub cdevice: SDL_ControllerDeviceEvent,
	pub padding: [u8;56],
}

#[link(name="SDL2")]
extern {
	// System
	pub fn SDL_Init(flags: u32) -> c_int;
	pub fn SDL_Quit();
	pub fn SDL_CreateWindow(title: *const c_char, x: c_int, y: c_int, w: c_int, h: c_int, flags: u32) -> *mut SDL_Window;
	pub fn SDL_DestroyWindow(window: *mut SDL_Window);
	pub fn SDL_GetWindowSize(window: *mut SDL_Window, w: *mut c_int, h: *mut c_int);
	pub fn SDL_GetWindowID(window: *mut SDL_Window) -> u32;
	pub fn SDL_GetError() -> *const c_char;
	pub fn SDL_GetPrefPath(org: *const c_char, app: *const c_char) -> *mut c_char;
	pub fn free(value: *mut c_void);
	// Events
	pub fn SDL_PollEvent(event: *mut SDL_Event) -> c_int;
	pub fn SDL_WaitEvent(event: *mut SDL_Event) -> c_int;
	pub fn SDL_GetKeyName(keycode: SDL_KeyCode) -> *const c_char;
	pub fn SDL_SetRelativeMouseMode(enable: bool) -> c_int;
	// Surface
	pub fn SDL_CreateRGBSurface(flags: u32, width: c_int, height: c_int, depth: c_int, rmask: u32, gmask: u32, bmask: u32, amask: u32) -> *mut SDL_Surface;
	pub fn SDL_CreateRGBSurfaceWithFormat(flags: u32, width: c_int, height: c_int, depth: c_int, format: u32) -> *mut SDL_Surface;
	pub fn SDL_CreateRGBSurfaceFrom(pixels: *const c_void, width: c_int, height: c_int, depth: c_int, pitch: c_int, rmask: u32, gmask: u32, bmask: u32, amask: u32) -> *mut SDL_Surface;
	pub fn SDL_CreateRGBSurfaceWithFormatFrom(pixels: *const c_void, width: c_int, height: c_int, depth: c_int, pitch: c_int, format: u32) -> *mut SDL_Surface;
	pub fn SDL_DuplicateSurface(surface: *mut SDL_Surface) -> *mut SDL_Surface;
	pub fn SDL_LoadBMP_RW(src: *mut SDL_RWops, freesrc: c_int) -> *mut SDL_Surface;
	pub fn SDL_RWFromFile(file: *const c_char, mode: *const c_char) -> *mut SDL_RWops;
	pub fn SDL_ConvertSurfaceFormat(src: *mut SDL_Surface, pixel_format: u32, flags: u32) -> *mut SDL_Surface;
	pub fn SDL_UpperBlit(src: *mut SDL_Surface, srcrect: *const SDL_Rect, dst: *mut SDL_Surface, dstrect: *mut SDL_Rect) -> c_int;
	pub fn SDL_FreeSurface(surface: *mut SDL_Surface);
	// Colour
	pub fn SDL_MapRGB(format: *const SDL_PixelFormat, r: u8, g: u8, b: u8) -> u32;
	pub fn SDL_MapRGBA(format: *const SDL_PixelFormat, r: u8, g: u8, b: u8, a: u8) -> u32;
	// Audio
	pub fn SDL_OpenAudio(desired: *mut SDL_AudioSpec, obtained: *mut SDL_AudioSpec) -> c_int;
	pub fn SDL_PauseAudio(pause: c_int);
	pub fn SDL_QueueAudio(dev: SDL_AudioDeviceID, data: *const c_void, len: u32) -> c_int;
	pub fn SDL_DequeueAudio(dev: SDL_AudioDeviceID, data: *const c_void, len: u32) -> u32;
	pub fn SDL_GetQueuedAudioSize(dev: SDL_AudioDeviceID) -> u32;
	// Controllers
	pub fn SDL_IsGameController(joystick_index: c_int) -> bool;
	pub fn SDL_GameControllerOpen(joystick_index: c_int) -> *mut SDL_GameController;
	pub fn SDL_GameControllerClose(controller: *mut SDL_GameController);
	pub fn SDL_GameControllerGetAttached(controller: *mut SDL_GameController) -> bool;
	pub fn SDL_GameControllerGetAxis(controller: *mut SDL_GameController, axis: SDL_GameControllerAxis) -> i16;
	pub fn SDL_GameControllerGetButton(controller: *mut SDL_GameController, button: SDL_GameControllerButton) -> u8;
	pub fn SDL_GameControllerGetJoystick(controller: *mut SDL_GameController) -> *mut SDL_Joystick;
	pub fn SDL_JoystickInstanceID(joystick: *mut SDL_Joystick) -> SDL_JoystickID;
	pub fn SDL_GameControllerFromInstanceID(joystick_id: SDL_JoystickID) -> *mut SDL_GameController;
	// pub fn SDL_GameControllerGetPlayerIndex(controller: *mut SDL_GameController) -> c_int;
	// pub fn SDL_GameControllerSetPlayerIndex(controller: *mut SDL_GameController, player_index: c_int);
	// OpenGL
	pub fn SDL_GL_SetAttribute(attribute: SDL_GLattr, value: c_int) -> c_int;
	pub fn SDL_GL_CreateContext(window: *mut SDL_Window) -> *mut SDL_GLContext;
	pub fn SDL_GL_DeleteContext(context: *mut SDL_GLContext);
	pub fn SDL_GL_MakeCurrent(window: *mut SDL_Window, context: *mut SDL_GLContext) -> c_int;
	pub fn SDL_GL_GetProcAddress(proc: *const c_char) -> *mut c_void;
	pub fn SDL_GL_SwapWindow(window: *mut SDL_Window);
	pub fn SDL_GL_SetSwapInterval(interval: c_int) -> c_int;
}
