//! This file is part of Flight.
//! 
//! Flight is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//! 
//! Flight is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//! You should have received a copy of the GNU General Public License along with Flight. If not, see <https://www.gnu.org/licenses/>.

#![allow(dead_code)]

use std::time::{Duration, Instant};

/// Framerate limiter.
pub struct FPS {
	delay: Duration,
	last: Instant,
}
impl FPS {
	pub fn new(fps: u32) -> Self {
		assert!(fps > 0);
		Self {
			delay: Duration::new(0, 1_000_000_000/fps),
			last: Instant::now(),
		}
	}
	pub fn tick(&mut self) {
		let now = Instant::now();
		let elapsed = now.duration_since(self.last);
		if elapsed > self.delay {
			// Low FPS, do nothing
			self.last = now;
		} else {
			std::thread::sleep(self.delay-elapsed);
			self.last = Instant::now();
		}
	}
}
