//! This file is part of Flight.
//! 
//! Flight is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//! 
//! Flight is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//! You should have received a copy of the GNU General Public License along with Flight. If not, see <https://www.gnu.org/licenses/>.

mod blend;
mod camera;
mod collide;
mod event;
pub mod ffi_gles;
mod ffi_keycodes;
pub mod ffi_sdl;
mod fps;
mod matrix3;
mod mesh;
mod mesh_data;
mod metrics;
mod offset_of;
pub mod pretty_print;
mod rgba;
mod rng;
mod rotation;
mod scenery;
mod shader;
mod texture;
mod tweak;
mod vector2;
mod vector3;
mod vertex;
mod window;

pub use blend::Blend;
pub use camera::{Anchor, Camera, CameraUniforms};
pub use collide::{CollisionMesh, RayHit};
pub use event::{Button, CONTROLLER_AXIS_COUNT, ControllerAxis, CONTROLLER_BUTTON_COUNT, ControllerButton, Event, Key, Modifiers};
pub use ffi_gles::GL;
pub use matrix3::Matrix3;
pub use mesh::{DynamicMesh, StaticMesh};
pub use mesh_data::MeshData;
pub use metrics::MetricUse;
pub use rgba::{rgba, RGBA};
pub use rng::RNG;
pub use rotation::Rotation;
pub use scenery::{SceneryBuilder, SceneryMesh, SceneryVertex};
pub use shader::{ActiveShader, Shader, Uniform};
pub use texture::{Repeat, Texture};
pub use tweak::Tweak;
pub use vector2::{f2, F2};
pub use vector3::{f3, F3};
pub use vertex::{Index, Primitive, Vertex, VertexBinding};
pub use window::Window;
