//! This file is part of Flight.
//! 
//! Flight is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//! 
//! Flight is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//! You should have received a copy of the GNU General Public License along with Flight. If not, see <https://www.gnu.org/licenses/>.

use std::fmt::{Display, Formatter, Result};

/// Print unsigned integers with thousands separators.
/// `assert_eq!(format!("{}", Thousands(15198)), "15,198");`
#[derive(Clone, Copy)]
pub struct Thousands(pub u64);
impl Display for Thousands {
	fn fmt(&self, f: &mut Formatter) -> Result {
		let mut buffer = [0 as u8; 25];
		let mut index = buffer.len();
		let Thousands(mut value) = *self;
		if value == 0 {
			index -= 1;
			buffer[buffer.len()-1] = b'0';
		} else {
			let mut counter = 0;
			while value > 0 {
				if counter == 3 {
					index -= 1;
					buffer[index] = b',';
					counter = 0;
				}
				index -= 1;
				buffer[index] = (value%10) as u8+b'0';
				value /= 10;
				counter += 1;
			}
		}
		f.write_str(std::str::from_utf8(&buffer[index..]).unwrap())
	}
}

#[cfg(test)]
mod tests {
	use super::*;

	#[test]
	fn thousands() {
		assert_eq!(format!("{}", Thousands(0)), "0");
		assert_eq!(format!("{}", Thousands(1)), "1");
		assert_eq!(format!("{}", Thousands(999)), "999");
		assert_eq!(format!("{}", Thousands(1000)), "1,000");
		assert_eq!(format!("{}", Thousands(99000)), "99,000");
		assert_eq!(format!("{}", Thousands(999000)), "999,000");
		assert_eq!(format!("{}", Thousands(2000000)), "2,000,000");
	}
}
