//! This file is part of Flight.
//! 
//! Flight is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//! 
//! Flight is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//! You should have received a copy of the GNU General Public License along with Flight. If not, see <https://www.gnu.org/licenses/>.

use super::ffi_gles::*;
use super::window::Window;

pub fn rgba(hex: u32) -> RGBA {
	RGBA {
		red: ((hex>>24)&0xff) as f32/255.0,
		green: ((hex>>16)&0xff) as f32/255.0,
		blue: ((hex>>8)&0xff) as f32/255.0,
		alpha: (hex&0xff) as f32/255.0,
	}
}

pub struct RGBA {
	pub red: f32,
	pub green: f32,
	pub blue: f32,
	pub alpha: f32,
}
impl RGBA {
	/// Fill screen with this colour.
	pub fn draw(&self, window: &mut Window) { unsafe {
		let gl = window.gl();
		(gl.ClearColor)(self.red, self.green, self.blue, self.alpha);
		(gl.Clear)(GL_COLOR_BUFFER_BIT);
	} }
	/// Linear interpolate from `self` to `other` by 0..1 value `factor`.
	pub fn lerp(self, other: Self, factor: f32) -> Self {
		Self {
			red: self.red+(other.red-self.red)*factor,
			green: self.green+(other.green-self.green)*factor,
			blue: self.blue+(other.blue-self.blue)*factor,
			alpha: self.alpha+(other.alpha-self.alpha)*factor,
		}
	}
}
