//! This file is part of Flight.
//! 
//! Flight is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
//! 
//! Flight is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//! You should have received a copy of the GNU General Public License along with Flight. If not, see <https://www.gnu.org/licenses/>.

use aircraft::Aircraft;
use camera::FollowCamera;
use collision::CollisionMesh;
use engine::RNG;
use entities::Entities;
use input::Input;
use shadow::Shadow;
use std::cell::RefCell;
use trail::Trail;

pub struct GameData {
	pub entities: Entities,
	pub floor_collision: CollisionMesh,
	pub input: Input<SeatData>,
	pub rng: RNG,
}

pub struct SeatData {
	pub aircraft: Aircraft,
	pub camera: FollowCamera,
	pub shadow: RefCell<Shadow>, // dirty hack to avoid borrowing mess
	pub trail_alpha: f32,
	pub trail_left: RefCell<Trail>, // dirty hack to avoid borrowing mess
	pub trail_right: RefCell<Trail>, // dirty hack to avoid borrowing mess
}
