class Game {
	// Return number of active players, can be zero
	foreign static count_players()
	// Return position of given player, must be within number of active players
	foreign static player_position(seat)
	// Set position of given player, must be within number of active players
	foreign static set_player_position(seat, position)
	// Return velocity of given player, must be within number of active players
	foreign static player_velocity(seat)
	// Set velocity of given player, must be within number of active players
	foreign static set_player_velocity(seat, velocity)
	// Return direction given player is facing, must be within number of active players
	foreign static player_direction(seat)
	// Set direction given player is facing, must be within number of active players
	foreign static set_player_direction(seat, direction)
	// Add entity to world
	// `mesh` is the graphic to use, see the constants in `Meshes`
	// `scale` is a `Vector3` giving the local scale of the graphic
	// `direction` is the direction the entity should face in
	// `translate` is the world location of the entity's centre
	foreign static add(mesh, scale, direction, translate)
	// Return local x axis for anything oriented to match the ground at a world
	// `position`
	// x is along the O'Neil cylinder, y is toward the cylinder's centre, z is
	// around the cylinder
	foreign static get_x(position)
	// See `get_x()`
	foreign static get_y(position)
	// See `get_x()`
	foreign static get_z(position)
	/*
	TODO: remove
	// Call the given `Fn` after some number of seconds, which can be zero
	foreign static delay(seconds, function)
	*/
	// Return a uniformly distributed random number from zero (inclusive) to one
	// (exclusive)
	foreign static random()
	// Return position a ray hit scenery or null if it didn't hit anything
	// `start` is the world origin point of the ray
	// `direction` is the world direction of the ray, it's length doesn't matter
	foreign static ray(start, direction)
	// Like `ray(_,_)`, but with a limited length.
	// `range` is the length of the ray, in terms of `direction`
	foreign static ray(start, direction, range)
	// Pause execution for given duration
	// Calling this from `Entity` tick functions works, but may become confusing!
	static wait(seconds) {
		// Queue current fiber to run later
		wait_internal(seconds, Fiber.current)
		// Stop fiber and end current execution
		Fiber.suspend()
	}
	// Implementation detail for `wait()`
	// Queues a fiber to resume after the given delay
	foreign static wait_internal(seconds, fiber)
}

class Meshes {
	static aircraft { 0 }
	static archway { 1 }
	static arrow { 1 }
	static finger { 2 }
	static gull { 3 }
	static ring { 4 }
	static shrub { 5 }
	static tree { 6 }
}

// Handle for a script-controlled object
foreign class Entity {
	// Delete the object
	foreign remove()
	// World unit vector giving direction the object is facing
	foreign direction
	foreign direction=(value)
	// Graphic from `Meshes` the object is shown as
	foreign mesh
	foreign mesh=(value)
	// World position of the object's centre
	foreign position
	foreign position=(value)
	// Scale of the object as a `Vector3`
	foreign scale
	foreign scale=(value)
	// Function to call everytime the object updates
	// This will be called at a fixed rate, likely less than the game framerate
	// See `entities.rs::Entities::FPS` for the exact interval
	// Set to `null` to call no function at all
	foreign tick=(function)
}

foreign class Vector3 {
	foreign static new()
	foreign static new(all)
	foreign static new(x, y, z)
	foreign x
	foreign y
	foreign z
	foreign x=(value)
	foreign y=(value)
	foreign z=(value)
	foreign -
	foreign +(other)
	foreign -(other)
	foreign *(other)
	foreign /(other)
	foreign cross(other)
	foreign dot(other)
	foreign length()
	foreign length2()
	foreign lerp(other, factor)
	foreign unitZero()
	foreign toString()
}
